import json
import logging
import os

from .uci import getValueOfUci

log = logging.getLogger('flask-backend')

SOURCES_STATUS_REPORT_FILE = "/tmp/reportStatus.json"

def getIPFormerSources():
    try:
        sources = {}
        config_dict = getValueOfUci('flexa_ip_former')
        """
        {'tpa400': {'level': 'PoE_Optimized'}, 'source1': {'name': 'Informacast', 'type': 'informacast', 'config': '{"url":"","idleaudio":"","idleaudio_format":"ULAW_8KHZ"}', 'volume': '90', 'prio': '1', 'enabled': 'true'}, 'source2': {'type': 'syn-apps', 'enabled': 'true', 'volume': '90', 'prio': '2', 'name': 'Synapps', 'config': '{"server":""}'}, 'source3': {}, 'source4': {}, 'source5': {}}
        """
        for key in config_dict:
            if key.startswith('source'):
                if all(k in config_dict[key] for k in ("name", "type", "config", "volume", "prio", "enabled")):
                    source = { "name": config_dict[key]["name"],
                               "type": config_dict[key]["type"],
                               "config": config_dict[key]["config"],
                               "volume": int(config_dict[key]["volume"]),
                               "prio": int(config_dict[key]["prio"]),
                               "enabled": True if config_dict[key]["enabled"] == 'true' else False }
                else:
                    source = {}
                sources[key] = source
        log.debug('flexa_ip_former ucis : {}'.format(sources))
    except Exception as e:
        log.error(e)
        raise e
    else:
        return sources

def getSourcesStatus():
    sourcesStatus = {}
    try:
        if os.path.isfile(SOURCES_STATUS_REPORT_FILE):
            with open(SOURCES_STATUS_REPORT_FILE) as json_file:
                sourcesStatus = json.load(json_file)
    except Exception as e:
        log.error(e)
        raise e
    else:
        return sourcesStatus