import logging
import os
import random
import socket
import string
import subprocess
import threading

from werkzeug.utils import secure_filename

from .exceptions import InvalidUploadedFileError, FileTooLargeError

log = logging.getLogger('werkzeug')

socketResponse = None
cv = threading.Condition() #conditionVariable

def listStoredFiles(path):
    fileList = []
    with os.scandir(path) as it:
        for entry in it:
            if not entry.name.startswith('.') and entry.is_file():
                fileList.append(entry)
    print(fileList)
    return fileList


def getFileInfo(fileList):
    infoFileList = []
    for file in fileList:
        infoFileList.append({"name": file.name, "size": file.stat().st_size})
    return infoFileList

def uploadFile(fileUploaded, fileID, validExtensions, destinationLoc):
    try:
        if fileID not in fileUploaded:
            raise InvalidUploadedFileError('Invalid file')
        uploadedFile = fileUploaded[fileID]
        # if user does not select file, browser also
        # submit an empty part without filename
        if uploadedFile.filename == '':
            raise InvalidUploadedFileError('Invalid filename')

        # check file extension
        if '.' in uploadedFile.filename and uploadedFile.filename.rsplit('.', 1)[1].lower() in validExtensions:
            try:
                uploadedFile.save(os.path.join(destinationLoc, uploadedFile.filename))
                return uploadedFile.filename + " installed."
            except OSError:
                raise InvalidUploadedFileError('Invalid file.')
            except Exception as e:
                raise e
        else:
            raise InvalidUploadedFileError('Format not supported.')

    except Exception as e:
        log.error("Error occurred while uploading fw file : {}".format(e))
        if str(e).startswith('413 Request Entity Too Large'):
            raise FileTooLargeError('REQUEST ENTITY TOO LARGE')
        else:
            raise e

def removeFiles(filePathList):
    try:
        counter = 0
        for filePath in filePathList:
            if os.path.isfile(filePath):
                os.remove(filePath)
                log.debug("File {} removed".format(filePath))
                counter += 1
    except Exception as e:
        log.error(e)
        raise e
    else:
        return counter


def playFile(audiofileName, ip_addr='127.0.0.1', port=60000):
    global socketResponse
    log.info("Requesting to play \"{}\"".format(audiofileName))
    socketResponse = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
        t = threading.Thread(target=waitForResult,
                             args=(sock,))
        t.start()
        message = "FILEPLAY={}\n".format(audiofileName)
        sock.sendto(bytes(message, "utf-8"), (ip_addr, port))
        with cv:
            while socketResponse is None:
                cv.wait()
        return socketResponse
    except Exception as e:
        log.error(e)
        raise e

def waitForResult(sock):
    global socketResponse
    response = sock.recvfrom(1024)
    socketResponse = response[0].decode("utf-8").strip('\r\n')
    with cv:
        cv.notify_all()

# def sendUDPMessage(message, ip_addr, port):
#     global socketResponse
#     try:
#         sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
#         t = threading.Thread(target=waitForResult,
#                              args=(sock,))
#         t.start()
#         sock.sendto(bytes(message, "utf-8"), (ip_addr, port))
#         with cv:
#             while socketResponse is None:
#                 cv.wait()
#             print("line 124")
#             print(socketResponse)
#     except Exception as e:
#         log.error(e)
#         raise e

def stopPlayingFile(ip_addr='127.0.0.1', port=60000):
    log.info("Stop playing audio file")
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
        message = "FILESTOP\n"
        sock.sendto(bytes(message, "utf-8"), (ip_addr, port))
    except Exception as e:
        log.error(e)
        raise e
