import logging
import hashlib

from .uci import getValueOfUci, setUciConfigs
from .constants import DEFAULT_WEBUI_LOGIN_USERNAME, DEFAULT_WEBUI_LOGIN_REALM, WEBUI_PASSWD_FILE_PATH
from .auth import generateNewSessionKey
from .exceptions import InvalidRequestBodyContentError, InvalidOldPasswordError

log = logging.getLogger('flask-backend')

def setWebUiPassword(pwdSettings):
    webUiPwdRequired = getValueOfUci('httpd', 'webserver', 'password_set')
    if 'pwd_old' in pwdSettings.keys():
        if webUiPwdRequired == 'false':
            log.error("pwd_old is in pwdSettings but password_set is disabled")
            raise InvalidRequestBodyContentError()
        else:
            try:
                isValid = currPasswordIsValid(pwdSettings['pwd_old'])
            except Exception as e:
                raise e
            else:
                if not isValid:
                    log.error("Old password is wrong")
                    raise InvalidOldPasswordError()
    else:
        if webUiPwdRequired == 'true':
            log.error("pwd_old not in pwdSettings but password_set is enabled")
            raise InvalidRequestBodyContentError()
    # at this stage, either pwd_old is valid or not required
    if 'pwd_set' not in pwdSettings.keys() or pwdSettings['pwd_set'] == '':
        # disable webui password
        try:
            enableWebUiPassword('false')
        except Exception as e:
            raise e
    else:
        # change or enable webui password
        try:
            setNewWebUIpassword(pwdSettings['pwd_set'])
            enableWebUiPassword('true')
        except Exception as e:
            raise e

def currPasswordIsValid(passwd, passwdFile=WEBUI_PASSWD_FILE_PATH, username=DEFAULT_WEBUI_LOGIN_USERNAME, realm=DEFAULT_WEBUI_LOGIN_REALM):
    strToDigest = username+":"+realm+":"+passwd
    try:
        digest = hashlib.md5(strToDigest.encode('utf-8')).hexdigest()
    except Exception as e:
        log.error("Can not calculate digest for the provided password: {}".format(e))
        raise
    try:
        f = open(passwdFile, 'r')
        content = f.read()
        f.close()
    except Exception as e:
        log.error("Error while reading passwd file contents: {}".format(e))
        raise e
    try:
        md5sum = ''
        fileLines = content.split('\n')
        for line in fileLines:
            tmpStr = username+':'+realm+':'
            if line.startswith(tmpStr):
                md5sum = line.split(':')[2]
                break
        if md5sum != '':
            return digest == md5sum
        else:
            return False
    except Exception as e:
        raise e

def setNewWebUIpassword(passwd, passwdFile=WEBUI_PASSWD_FILE_PATH, username=DEFAULT_WEBUI_LOGIN_USERNAME, realm=DEFAULT_WEBUI_LOGIN_REALM):
    strToDigest = username + ":" + realm + ":" + passwd
    try:
        digest = hashlib.md5(strToDigest.encode('utf-8')).hexdigest()
    except Exception as e:
        log.error("Can not calculate digest for the provided password : {}".format(e))
        raise e
    try:
        f = open(passwdFile, 'w')
        strToWrite = username+":"+realm+":"+digest
        f.write(strToWrite)
        f.close()
        # generate new session Key
        generateNewSessionKey()
    except Exception as e:
        log.error("Error while writting new passwd in file: {}".format(e))
        raise e

def enableWebUiPassword(flag_value):
    try:
        # set webUI password flag to true
        setUciConfigs({'httpd.webserver.password_set': flag_value}, restartServices=False)
    except Exception as e:
        raise e